from earthdiagnostics import Utils, cdftools, cdo, TempFile
from earthdiagnostics.basins import Basins
from autosubmit.config.log import Log
import os
import shutil
import traceback
from nco import NCOException


class Heat(object):

    @staticmethod
    def mixed_layer_content(input_file, mltost_file, output_file):
        """
        Compute mixed layer heat and salt content

        Created in February 2012      Author : vguemas@ic3.cat

        :param mltost_file: imput mltost file
        :param input_file: input grid_T file name
        :param output_file: output file name  (=> 2D x-y )
        :return:
        """

        nco = Utils.nco

        input_scratch = TempFile.get()
        shutil.copy(input_file, input_scratch)
        nco.ncks(input=mltost_file, output=input_scratch, options='-A -v mlotst')

        ntime = int(cdo.ntime(input=input_scratch)[0])
        files = list()

        for time in range(ntime):
            Log.info('Running time {0}', time)
            temp = TempFile.get()
            nco.ncks(input=input_scratch, output=temp, options='-O -d time,{0}'.format(time))
            Log.info('Computing salt content')
            cdftools.run('cdfmxlheatc', input=temp, options=['-full'])
            Utils.move_file('mxlheatc.nc', temp)
            files.append(temp)

        temp = TempFile.get()
        cdo.cat(input=' '.join(files), output=temp,)
        nco.ncks(input=input_scratch, output=temp, options='-A -v time')

        for temp_file in files:
            os.remove(temp_file)
        os.remove(input_scratch)

        Utils.rename_variables(temp, {'x': 'i', 'y': 'j', 'somxlheatc': 'mlotsthc'}, False, True)
        Utils.setminmax(temp, 'mlotsthc')
        Utils.move_file(temp, output_file)

    @staticmethod
    def layer(input_file, output_file, upper, lower):
        """
        Pointwise Ocean Heat Content in a specified ocean thickness (J/m-2)

        Created in June 2012      Author : isabel.andreu-burillo@ic3.cat
        May 2014 - Virginie Guemas - Way around the bc that does not work on moore

        :param input_file: input grid_T file nam
        :param output_file: output file name  (=> 2D x-y )
        :param upper: upper depth of the layer (in meters)
        :param lower: lower depth of the layer (in meters)
        :return:
        """

        nco = Utils.nco

        temp = TempFile.get()
        temp2 = TempFile.get()
        heatc_sl_out = TempFile.get()
        heatc_sl_top = TempFile.get()
        heatc_sl_bottom = TempFile.get()
        heatc_sl_invert = TempFile.get()
        e3tfile = TempFile.get()

        nco.ncap2(input='mesh_zgr.nc', output=e3tfile, options='-v -O -s "heatc_sl=tmask*e3t"'),
        nco.ncrename(input=e3tfile, output=e3tfile, options='-d t,time -d z,deptht')

        nco.ncks(input=input_file, output=temp, options='-O -v votemper')
        nco.ncrename(input=temp, output=temp, options='-v votemper,heatc_sl')
        cdo.mul(input=' '.join([temp, e3tfile]), output=heatc_sl_out)
        os.remove(e3tfile)
        nco.ncks(input=heatc_sl_out, output=temp, options='-O -d deptht,{0}.0,{1}.0'.format(upper, lower))

        # perform the integration of ohc down to that level (main contribution)
        nco.ncap2(input=temp, output=heatc_sl_top, options='-O -s "heatc_sl=heatc_sl.total($deptht)"')

        # now extract a few levels below, to compute the residual ohc
        # obtain the weight for the extra level containing the 300 m
        # deptht in the gridT files is positive
        # weight = (300.0 - depth_top)/(depth_bottom - depth_top)
        # and add the thickness down to 300 m in the next layer
        nco.ncpdq(input=heatc_sl_out, output=heatc_sl_invert, options='-a "-deptht"')
        nco.ncks(input=heatc_sl_invert, output=temp2, options='-O -d deptht,0,0')
        Utils.rename_variable(temp2, 'deptht', 'layerthcknss')
        shutil.move(temp2, heatc_sl_invert)

        nco.ncks(input=heatc_sl_out, output=heatc_sl_bottom,
                 options='-O -d deptht,{0}.0,{1}.0'.format(lower, lower + 200))
        nco.ncks(input=heatc_sl_bottom, output=heatc_sl_bottom, options='-O -d deptht,0,0')
        Utils.rename_variable(heatc_sl_bottom, 'deptht', 'layerthcknss')

        os.remove(heatc_sl_out)

        # Here, add the residual contribution, before adding it to the main contribution
        nco.ncbo(input=' '.join([heatc_sl_bottom, heatc_sl_invert]), output=temp,
                 options='-A --op_typ=sub -v layerthcknss')

        Utils.rename_variable(temp, 'layerthcknss', 'heatc_sl')
        nco.ncap2(input=heatc_sl_invert, output=temp2, options='-s "heatc_sl=({0:.1f} - layerthcknss)"'.format(lower))
        os.remove(heatc_sl_invert)
        nco.ncbo(input=' '.join([temp, temp2]), output=temp,  options='--op_typ=/ -v heatc_sl')
        os.remove(temp2)
        nco.ncrename(input=temp, output=temp, options='-v heatc_sl,factor')
        nco.ncks(input=temp, output=heatc_sl_bottom, options='-A -v factor')
        nco.ncap2(input=heatc_sl_bottom, output=heatc_sl_bottom, options='-O -s "heatc_sl=(factor * heatc_sl)"')
        nco.ncwa(input=heatc_sl_bottom, output=heatc_sl_bottom, options='-O -a deptht')
        nco.ncbo(input=' '.join([heatc_sl_top, heatc_sl_bottom]), output=temp,  options='--op_typ=+ -v heatc_sl')
        os.remove(heatc_sl_top)
        os.remove(heatc_sl_bottom)
        nco.ncap2(input=temp, output=temp, options='-s "heatc_sl=1020.0*4000*heatc_sl"')

        Utils.setminmax(temp, ['heatc_sl'])

        shutil.move(temp, output_file)
        os.remove(temp)

    @staticmethod
    def total(input_file, mlotst_file, output_file, basin=Basins.Global.shortname, mixed_layer=0, upper=None, lower=None):
        """
        Compute the total ocean heat extent

        Created in May 2012           Author : vguemas@ic3.cat

        :param input_file: input temperature file name
        :param output_file: output file name  ( => 2D x-y
        :param basin: basin name (default: global)
        :param mixed_layer: mixed layer (1=only, 0=included, -1=without)
        :param upper: upper level of the layer (optional) Default : top
        :param lower: lower level of the layer (optional) Default : bottom
        """

        nco = Utils.nco
        temp = TempFile.get()
        temp2 = TempFile.get()

        shutil.copy(input_file, temp)
        nco.ncks(input=mlotst_file, output=temp, options='-A -v mlotst')

        if basin == Basins.NorthAtlantic.shortname:
            para = "{0} 0 0 10 65"
        elif basin == Basins.TropicalAtlantic.shortname:
            para = "{0} 0 0 -30 30"
        elif basin == Basins.NorthPacific.shortname:
            para = "{0} 0 0 10 70"
        elif basin == Basins.TropicalPacific.shortname:
            para = "{0} 0 0 -30 30"
        elif basin == Basins.Arctic.shortname:
            para = "{0} 0 0 65 90"
        elif basin == Basins.Antarctic.shortname:
            para = "{0} 0 0 -90 -60"
        elif basin == Basins.TropicalIndian.shortname:
            para = "{0} 0 0 -30 30"
        elif basin == Basins.Global.shortname:
            para = "{0} 0 0 0 0"
        else:
            raise Exception('Basin {0} not recognized in Heat.total'.format(basin))

        #
        # Compute ohc
        #
        para = "-full" + para.format(mixed_layer)
        para = para.split()
        para = para[1:]
        para.append(upper)
        para.append(lower)
        para.append(str(mixed_layer))
        shell_output = cdftools.run('cdfheatc', options=para, input=temp)

        nco.ncks(input=temp, output=temp2, options='-O -v time')
        output = nco.openCdf(temp2)
        thc = output.createVariable('thc', float, 'time')
        uhc = output.createVariable('uhc', float, 'time')
        time = 0
        for lines in shell_output:
            if not lines:
                continue

            for line in lines.split('\n'):
                line = line.lstrip()
                if line.startswith("Total Heat content "):
                    Log.result(line)
                    thc[time] = line[line.index(':')+1: line.index('Joules')]
                elif line.startswith("Total Heat content/volume"):
                    Log.user_warning(line)
                    uhc[time] = line[line.index(':')+1: line.index('Joules')]
                    time += 1
                elif line.startswith('TIME : '):
                    Log.info(line)

        output.close()
        Utils.setminmax(temp2, ['thc', 'uhc'])
        Utils.move_file(temp2, output_file)


def main():
    try:
        Heat.layer("ORCA1_MM_19601101_19610228_grid_T.nc", "out.nc", 300, 2000)
    except NCOException as ex:
        Log.error('Exception {0}: stderr: {1}, stdout: {2}',  ex.msg, ex.stderr, ex.stdout)
        Log.error(traceback.format_exc())


if __name__ == "__main__":
    main()
